"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.copyTx = exports.cloneTx = void 0;
const transaction_1 = __importDefault(require("blockweave/dist/lib/transaction"));
const cloneTx = async (tx, blockweave, wallet, data = "") => {
    const ntx = await transaction_1.default.create(blockweave, {
        format: tx.format,
        owner: tx.owner,
        target: tx.target,
        quantity: tx.quantity,
        last_tx: tx.last_tx,
        data,
        reward: tx.reward,
    }, wallet);
    // map tags
    const tags = tx.get("tags");
    tags.forEach((tag) => {
        ntx.addTag(tag.get("name", { decode: true, string: true }), tag.get("value", { decode: true, string: true }));
    });
    ntx.setSignature({
        id: tx.id,
        signature: tx.signature,
        owner: wallet.n,
    });
    if (blockweave.constructor.name === "Arweave") {
        await blockweave.transactions.post(ntx);
        return ntx.id;
    }
    const uploader = await blockweave.transactions.getUploader(ntx);
    while (!uploader.isComplete) {
        try {
            await uploader.uploadChunk();
        }
        catch (e) {
            const { msg } = uploader.lastResponseError;
            if (msg) {
                console.error(uploader.lastResponseError);
                return null;
            }
            console.error(uploader.lastResponseError);
        }
    }
    return ntx.id;
};
exports.cloneTx = cloneTx;
const copyTx = async (tx, blockweave, wallet, data = "", extraTags = []) => {
    const localTx = await blockweave.createTransaction({
        data,
    }, wallet);
    // map tags
    const tags = tx.get("tags");
    // add support for tag overwrite
    const fTags = tags.filter((tag) => {
        const name = tag.get("name", { decode: true, string: true });
        const isOverwritten = extraTags.some((t) => t.name === name);
        if (isOverwritten) {
            return false;
        }
        return true;
    });
    fTags.forEach((tag) => {
        localTx.addTag(tag.get("name", { decode: true, string: true }), tag.get("value", { decode: true, string: true }));
    });
    if (extraTags.length) {
        // Add extra tags
        extraTags.forEach((tag) => {
            localTx.addTag(tag.name, tag.value);
        });
    }
    await blockweave.transactions.sign(localTx, wallet);
    const uploader = await blockweave.transactions.getUploader(localTx);
    while (!uploader.isComplete) {
        try {
            await uploader.uploadChunk();
        }
        catch (e) {
            const { msg } = uploader.lastResponseError;
            if (msg) {
                console.error(uploader.lastResponseError);
                return null;
            }
            console.error(uploader.lastResponseError);
        }
    }
    return localTx.id;
};
exports.copyTx = copyTx;
